"""Helper functions for NetgearPy."""


def time_to_minutes(source: str) -> int:
    """Convert a time string to minutes."""
    parts = source.split(":")
    if parts[0] == "--" and parts[1] == "--":
        return 0
    hours = int(parts[0])
    minutes = int(parts[1])
    return hours * 60 + minutes


def statistics_to_data(source: str) -> tuple[float, float]:
    """Convert a statistics string to a dictionary."""
    parts = source.split("/")
    return float(parts[0]), float(parts[1])



"""Constants for NetgearPy."""

from netgearpy.models import Service

ENVELOPE = """<!--?xml version="1.0" encoding= "UTF-8" ?-->
<v:Envelope
xmlns:v="http://schemas.xmlsoap.org/soap/envelope/">
<v:Header>
<SessionID>A7D88AE69687E58D9A00</SessionID>
</v:Header>
<v:Body>
{}
</v:Body>
</v:Envelope>"""

EMPTY_BODY = """<M1:{} xmlns:M1="urn:NETGEAR-ROUTER:service:{}:1" />"""

LOGIN_BODY = """<M1:SOAPLogin xmlns:M1="urn:NETGEAR-ROUTER:service:DeviceConfig:1">
<Username>{}</Username><Password>{}</Password></M1:SOAPLogin>"""


def _get_empty_body(service: Service, object_type: str) -> str:
    """Get an empty body for a SOAP request."""
    return EMPTY_BODY.format(object_type, service)


GET_ATTACHED_DEVICES_BODY = _get_empty_body(Service.DEVICE_INFO, "GetAttachDevice")

GET_INFO_BODY = _get_empty_body(Service.DEVICE_INFO, "GetInfo")

GET_TRAFFIC_METER_STATISTICS_BODY = _get_empty_body(
    Service.DEVICE_CONFIG, "GetTrafficMeterStatistics"
)

GET_BLOCK_DEVICE_ENABLE_BODY = _get_empty_body(
    Service.DEVICE_CONFIG, "GetBlockDeviceEnableStatus"
)

GET_SYSTEM_INFO_BODY = """<M1:GetSystemInfo xsi:nil="true" />"""

IS_TRAFFIC_METER_ENABLED_BODY = _get_empty_body(
    Service.DEVICE_CONFIG, "GetTrafficMeterEnabled"
)

IS_GUEST_ACCESS_ENABLED_BODY = _get_empty_body(
    Service.WLAN_CONFIGURATION, "GetGuestAccessEnabled"
)

IS_5G_GUEST_ACCESS_ENABLED_BODY = _get_empty_body(
    Service.WLAN_CONFIGURATION, "Get5G1GuestAccessEnabled"
)

IS_QOS_ENABLED_BODY = _get_empty_body(Service.ADVANCED_QOS, "GetQoSEnableStatus")

IS_SMART_CONNECT_ENABLED = _get_empty_body(
    Service.WLAN_CONFIGURATION, "IsSmartConnectEnabled"
)

IS_PARENTAL_CONTROL_ENABLED_BODY = """<GetEnableStatus></GetEnableStatus>"""

GET_ETHERNET_LINK_STATUS_BODY = """<M1:GetEthernetLinkStatus xsi:nil="true" />"""



"""Asynchronous Python client for Netgear routers."""

from __future__ import annotations

import asyncio
from dataclasses import dataclass
from importlib import metadata
import logging
from typing import Any, Self
import xml.etree.ElementTree as ET

from aiohttp import ClientSession
from aiohttp.hdrs import METH_GET, METH_POST
from yarl import URL

from netgearpy.const import (
    ENVELOPE,
    GET_ATTACHED_DEVICES_BODY,
    GET_BLOCK_DEVICE_ENABLE_BODY,
    GET_ETHERNET_LINK_STATUS_BODY,
    GET_INFO_BODY,
    GET_SYSTEM_INFO_BODY,
    GET_TRAFFIC_METER_STATISTICS_BODY,
    IS_5G_GUEST_ACCESS_ENABLED_BODY,
    IS_GUEST_ACCESS_ENABLED_BODY,
    IS_PARENTAL_CONTROL_ENABLED_BODY,
    IS_QOS_ENABLED_BODY,
    IS_SMART_CONNECT_ENABLED,
    IS_TRAFFIC_METER_ENABLED_BODY,
    LOGIN_BODY,
)
from netgearpy.models import (
    AdvancedQoSAction,
    AttachedDevice,
    CurrentSettings,
    DeviceConfigAction,
    DeviceInfo,
    DeviceInfoAction,
    ParentalControlAction,
    Service,
    SystemInfo,
    TrafficMeterStatistics,
    WanEthernetLinkConfigAction,
    WlanConfigurationAction,
)

VERSION = metadata.version(__package__)

_LOGGER = logging.getLogger(__package__)


@dataclass
class NetgearClient:
    """Main class for handling connections with a Netgear router."""

    host: str
    session: ClientSession | None = None
    request_timeout: int = 10
    _close_session: bool = False
    _soap_port: int | None = None
    _login_method: int | None = None
    _lock = asyncio.Lock()

    async def _request(
        self,
        url: str,
        *,
        method: str = METH_GET,
        data: str | None = None,
        headers: dict[str, Any] | None = None,
    ) -> str:
        """Handle a request to a Netgear router."""
        base_headers = {
            "User-Agent": f"Netgearpy/{VERSION}",
            "Accept": "application/xml",
        }

        if self.session is None:
            self.session = ClientSession()
            self._close_session = True

        if headers is None:
            headers = {}

        async with asyncio.timeout(self.request_timeout):
            response = await self.session.request(
                method,
                url,
                headers=base_headers | headers,
                data=data,
            )

        return await response.text()

    async def _get(self, uri: str) -> str:
        """Handle a GET request to Netgear."""
        return await self._request(uri, method=METH_GET)

    async def get_current_setting(self) -> CurrentSettings:
        """Get the current settings from the Netgear router."""
        url = URL.build(scheme="http", host=self.host, path="/currentsetting.htm")
        response = await self._get(str(url))
        res: dict[str, Any] = {}
        for entry in response.split("\n"):
            if entry:
                split_entry = entry.split("=")
                res[split_entry[0]] = split_entry[1]
        result = CurrentSettings.from_dict(res)
        self._soap_port = result.https_port
        self._login_method = result.login_method
        return result

    async def _post_xml(
        self, service: str, action: str, body: str
    ) -> dict[str, str | None]:
        """Post XML data to the Netgear router."""
        if self._soap_port is None or self._login_method is None:
            await self.get_current_setting()
        https = self._soap_port in [443, 5555]
        url = URL.build(
            scheme="https" if https else "http",
            host=self.host,
            path="/soap/server_sa/",
            port=self._soap_port,
        )
        headers = {
            "SOAPAction": f"urn:NETGEAR-ROUTER:service:{service}:1#{action}",
        }
        body_str = ENVELOPE.format(body)
        async with self._lock:
            await asyncio.sleep(0.3)
            if service != DeviceConfigAction.LOGIN:
                _LOGGER.debug(
                    "Posting XML to %s with action `%s` and body: %s",
                    url,
                    f"urn:NETGEAR-ROUTER:service:{service}:1#{action}",
                    body_str,
                )
            response = await self._request(
                str(url), method=METH_POST, data=body_str, headers=headers
            )
        _LOGGER.debug("Response from %s: %s", url, response)
        response_dict = {}
        root = ET.fromstring(response)  # noqa: S314
        for item in root.iter():
            if item.tag != "{http://schemas.xmlsoap.org/soap/envelope/}Body":
                continue
            for child in item:
                for subchild in child:
                    response_dict[subchild.tag] = subchild.text
        return response_dict

    async def login(self, username: str, password: str) -> None:
        """Login to the Netgear router."""
        await self._post_xml(
            Service.DEVICE_CONFIG,
            DeviceConfigAction.LOGIN,
            LOGIN_BODY.format(username, password),
        )

    async def get_attached_devices(self) -> list[AttachedDevice]:
        """Get the attached devices from the Netgear router."""
        response = await self._post_xml(
            Service.DEVICE_INFO,
            DeviceInfoAction.GET_ATTACHED_DEVICES,
            GET_ATTACHED_DEVICES_BODY,
        )
        device_string = response["NewAttachDevice"]
        if not device_string:
            return []
        return [
            AttachedDevice.from_string(entry) for entry in device_string.split("@")[1::]
        ]

    async def is_parental_control_enabled(self) -> bool:
        """Check if parental control is enabled."""
        response = await self._post_xml(
            Service.PARENTAL_CONTROL,
            ParentalControlAction.GET_ENABLE_STATUS,
            IS_PARENTAL_CONTROL_ENABLED_BODY,
        )
        return response.get("ParentalControl") == "1"

    async def is_block_device_enabled(self) -> bool:
        """Check if block device is enabled."""
        response = await self._post_xml(
            Service.DEVICE_CONFIG,
            DeviceConfigAction.GET_BLOCK_DEVICE_ENABLE_STATUS,
            GET_BLOCK_DEVICE_ENABLE_BODY,
        )
        return response.get("ParentalControl") == "1"

    async def get_device_info(self) -> DeviceInfo:
        """Get device information from the Netgear router."""
        response = await self._post_xml(
            Service.DEVICE_INFO, DeviceInfoAction.GET_INFO, GET_INFO_BODY
        )
        return DeviceInfo.from_dict(response)

    async def get_traffic_meter_statistics(self) -> TrafficMeterStatistics:
        """Get traffic meter statistics from the Netgear router."""
        response = await self._post_xml(
            Service.DEVICE_CONFIG,
            DeviceConfigAction.GET_TRAFFIC_METER_STATISTICS,
            GET_TRAFFIC_METER_STATISTICS_BODY,
        )
        return TrafficMeterStatistics.from_dict(response)

    async def get_system_info(self) -> SystemInfo:
        """Get system information from the Netgear router."""
        response = await self._post_xml(
            Service.DEVICE_INFO, DeviceInfoAction.GET_SYSTEM_INFO, GET_SYSTEM_INFO_BODY
        )
        return SystemInfo.from_dict(response)

    async def is_traffic_meter_enabled(self) -> bool:
        """Check if the traffic meter is enabled."""
        response = await self._post_xml(
            Service.DEVICE_CONFIG,
            DeviceConfigAction.GET_TRAFFIC_METER_ENABLED,
            IS_TRAFFIC_METER_ENABLED_BODY,
        )
        return response.get("NewTrafficMeterEnable") == "1"

    async def is_guest_access_enabled(self) -> bool:
        """Check if guest access is enabled."""
        response = await self._post_xml(
            Service.WLAN_CONFIGURATION,
            WlanConfigurationAction.GET_GUEST_ACCESS_ENABLED,
            IS_GUEST_ACCESS_ENABLED_BODY,
        )
        return response.get("NewGuestAccessEnabled") == "1"

    async def is_5g_guest_access_enabled(self) -> bool:
        """Check if 5G guest access is enabled."""
        response = await self._post_xml(
            Service.WLAN_CONFIGURATION,
            WlanConfigurationAction.GET_5G1_GUEST_ACCESS_ENABLED_2,
            IS_5G_GUEST_ACCESS_ENABLED_BODY,
        )
        return response.get("NewGuestAccessEnabled") == "1"

    async def is_qos_enabled(self) -> bool:
        """Check if QoS is enabled."""
        response = await self._post_xml(
            Service.ADVANCED_QOS,
            AdvancedQoSAction.GET_QOS_ENABLE_STATUS,
            IS_QOS_ENABLED_BODY,
        )
        return response.get("NewQoSEnableStatus") == "1"

    async def is_smart_connect_enabled(self) -> bool:
        """Check if Smart Connect is enabled."""
        response = await self._post_xml(
            Service.WLAN_CONFIGURATION,
            WlanConfigurationAction.IS_SMART_CONNECT_ENABLED,
            IS_SMART_CONNECT_ENABLED,
        )
        return response.get("NewSmartConnectEnable") == "1"

    async def get_ethernet_link_status(self) -> str:
        """Get the ethernet link status from the Netgear router."""
        response = await self._post_xml(
            Service.WAN_ETHERNET_LINK_CONFIG,
            WanEthernetLinkConfigAction.GET_ETHERNET_LINK_STATUS,
            GET_ETHERNET_LINK_STATUS_BODY,
        )
        assert response["NewEthernetLinkStatus"] is not None  # noqa: S101
        return response["NewEthernetLinkStatus"]

    async def close(self) -> None:
        """Close open client session."""
        if self.session and self._close_session:
            await self.session.close()

    async def __aenter__(self) -> Self:
        """Async enter.

        Returns
        -------
            The NetgearClient object.

        """
        return self

    async def __aexit__(self, *_exc_info: object) -> None:
        """Async exit.

        Args:
        ----
            _exc_info: Exec type.

        """
        await self.close()



"""NetgearPy models for device settings."""

from __future__ import annotations

from dataclasses import dataclass, field
from enum import IntEnum, StrEnum
from typing import Any

from mashumaro import field_options
from mashumaro.mixins.dict import DataClassDictMixin

from netgearpy.helpers import statistics_to_data, time_to_minutes


class Service(StrEnum):
    """Enum for service types."""

    DEVICE_CONFIG = "DeviceConfig"
    TIME = "Time"
    LAN_CONFIG_SECURITY = "LANConfigSecurity"
    WAN_IP_CONNECTION = "WANIPConnection"
    WAN_ETHERNET_LINK_CONFIG = "WANEthernetLinkConfig"
    PARENTAL_CONTROL = "ParentalControl"
    DEVICE_INFO = "DeviceInfo"
    ADVANCED_QOS = "AdvancedQoS"
    WLAN_CONFIGURATION = "WLANConfiguration"


class DeviceConfigAction(StrEnum):
    """Enum for device config actions."""

    LOGIN = "SOAPLogin"
    LOGOUT = "SOAPLogout"
    REBOOT = "Reboot"
    CHECK_NEW_FIRMWARE = "CheckNewFirmware"
    CHECK_APP_NEW_FIRMWARE = "CheckAppNewFirmware"
    UPDATE_NEW_FIRMWARE = "UpdateNewFirmware"
    CONFIGURATION_STARTED = "ConfigurationStarted"
    CONFIGURATION_FINISHED = "ConfigurationFinished"
    GET_INFO = "GetInfo"
    GET_TIME_ZONE_INFO = "GetTimeZoneInfo"

    GET_DEVICE_LIST_ALL = "GetDeviceListAll"
    GET_BLOCK_DEVICE_ENABLE_STATUS = "GetBlockDeviceEnableStatus"
    SET_BLOCK_DEVICE_ENABLE = "SetBlockDeviceEnable"
    ENABLE_BLOCK_DEVICE_FOR_ALL = "EnableBlockDeviceForAll"
    SET_BLOCK_DEVICE_BY_MAC = "SetBlockDeviceByMAC"

    GET_TRAFFIC_METER_ENABLED = "GetTrafficMeterEnabled"
    GET_TRAFFIC_METER_OPTIONS = "GetTrafficMeterOptions"
    GET_TRAFFIC_METER_STATISTICS = "GetTrafficMeterStatistics"
    ENABLE_TRAFFIC_METER = "EnableTrafficMeter"


class TimeAction(StrEnum):
    """Enum for time actions."""

    GET_INFO = "GetInfo"


class LanConfigSecurityAction(StrEnum):
    """Enum for LAN config security actions."""

    GET_INFO = "GetInfo"


class WanIpConnectionAction(StrEnum):
    """Enum for WAN IP connection actions."""

    GET_INFO = "GetInfo"
    GET_PORT_MAPPING_INFO = "GetPortMappingInfo"
    GET_CONNECTION_TYPE_INFO = "GetConnectionTypeInfo"
    GET_INTERNET_PORT_INFO = "GetInternetPortInfo"


class WanEthernetLinkConfigAction(StrEnum):
    """Enum for WAN Ethernet link config actions."""

    GET_ETHERNET_LINK_STATUS = "GetEthernetLinkStatus"


class ParentalControlAction(StrEnum):
    """Enum for parental control actions."""

    LOGIN = "Authenticate"
    GET_ENABLE_STATUS = "GetEnableStatus"
    ENABLE_PARENTAL_CONTROL = "EnableParentalControl"


class DeviceInfoAction(StrEnum):
    """Enum for device info actions."""

    GET_INFO = "GetInfo"
    GET_SUPPORT_FEATURE_LIST_XML = "GetSupportFeatureListXML"
    GET_ATTACHED_DEVICES = "GetAttachDevice"
    GET_ATTACHED_DEVICES_2 = "GetAttachDevice2"
    SET_NETGEAR_DEVICE_NAME = "SetNetgearDeviceName"
    GET_SYSTEM_LOGS = "GetSystemLogs"
    GET_SYSTEM_INFO = "GetSystemInfo"
    GET_SYSTEM_UPTIME = "GetSysUpTime"


class AdvancedQoSAction(StrEnum):
    """Enum for advanced QoS actions."""

    SET_OOKLA_SPEEDTEST_START = "SetOOKLASpeedTestStart"
    GET_OOKLA_SPEEDTEST_RESULT = "GetOOKLASpeedTestResult"
    GET_QOS_ENABLE_STATUS = "GetQoSEnableStatus"
    SET_QOS_ENABLE_STATUS = "SetQoSEnableStatus"
    GET_BANDWIDTH_CONTROL_OPTIONS = "GetBandwidthControlOptions"
    SET_BANDWIDTH_CONTROL_OPTIONS = "SetBandwidthControlOptions"


class WlanConfigurationAction(StrEnum):
    """Enum for WLAN configuration actions."""

    GET_GUEST_ACCESS_ENABLED = "GetGuestAccessEnabled"  # 2.4G-1 R7800/R8000
    GET_5G1_GUEST_ACCESS_ENABLED = "Get5GGuestAccessEnabled"  # 5G-1 R7800
    GET_5G1_GUEST_ACCESS_ENABLED_2 = "Get5G1GuestAccessEnabled"  # 5G-1 R8000
    GET_5G_GUEST_ACCESS_DISABLED_2 = "Get5GGuestAccessEnabled2"  # 5G-2 R8000
    SET_GUEST_ACCESS_ENABLED = "SetGuestAccessEnabled"  # 2.4G-1 R7800
    SET_GUEST_ACCESS_ENABLED_2 = "SetGuestAccessEnabled2"  # 2.4G-1 R8000
    SET_5G1_GUEST_ACCESS_ENABLED = "Set5GGuestAccessEnabled"  # 5G-1 R7800
    SET_5G1_GUEST_ACCESS_ENABLED_2 = "Set5G1GuestAccessEnabled2"  # 5G-1 R8000
    SET_5G_GUEST_ACCESS_ENABLED_2 = "Set5GGuestAccessEnabled2"  # 5G-2 R8000
    IS_SMART_CONNECT_ENABLED = "IsSmartConnectEnabled"
    SET_SMART_CONNECT_ENABLED = "SetSmartConnectEnable"
    GET_AVAILABLE_CHANNEL = "GetAvailableChannel"
    GET_CHANNEL_INFO = "GetChannelInfo"
    GET_5G_CHANNEL_INFO = "Get5GChannelInfo"
    GET_5G1_CHANNEL_INFO = "Get5G1ChannelInfo"
    SET_CHANNEL = "SetChannel"
    SET_5G_CHANNEL = "Set5GChannel"
    SET_5G1_CHANNEL = "Set5G1Channel"


@dataclass
class CurrentSettings(DataClassDictMixin):
    """Represents the current setting of a device."""

    firmware_version: str = field(metadata=field_options(alias="Firmware"))
    model: str = field(metadata=field_options(alias="Model"))
    login_method: int = field(metadata=field_options(alias="LoginMethod"))
    https_port: int = field(metadata=field_options(alias="SOAP_HTTPs_Port"))

    @classmethod
    def __pre_deserialize__(cls, d: dict[str, Any]) -> dict[str, Any]:
        """Pre deserialize hook."""
        return {
            **d,
            "LoginMethod": int(float(d["LoginMethod"])),
        }


class ConnectionType(StrEnum):
    """Enum for connection types."""

    WIRED = "wired"
    WIRELESS = "wireless"


@dataclass(kw_only=True)
class AttachedDevice:
    """Represents an attached device."""

    ip_address: str
    hostname: str | None
    mac_address: str
    connection_type: ConnectionType | None
    link_speed: int | None
    signal_strength: int | None
    blocked: bool | None

    @classmethod
    def from_string(cls, source_string: str) -> AttachedDevice:
        """Pre deserialize hook."""
        parts = source_string.split(";")
        ip_address = parts[1]
        hostname = None
        if parts[2] != "--":
            hostname = parts[2]
        mac_address = parts[3]
        connection_type = None
        link_speed = None
        signal_strength = None
        blocked = None
        if len(parts) >= 7:
            connection_type = ConnectionType(parts[4])
            if parts[5] != "":
                link_speed = int(parts[5])
            signal_strength = int(parts[6])
            if len(parts) >= 8:
                blocked = parts[7] != "Allow"
        return AttachedDevice(
            ip_address=ip_address,
            hostname=hostname,
            mac_address=mac_address,
            connection_type=connection_type,
            link_speed=link_speed,
            signal_strength=signal_strength,
            blocked=blocked,
        )


class DeviceMode(IntEnum):
    """Enum for device modes."""

    ROUTER = 0
    ACCESS_POINT = 1


@dataclass
class DeviceInfo(DataClassDictMixin):
    """Represents the device information."""

    model: str = field(metadata=field_options(alias="ModelName"))
    serial_number: str = field(metadata=field_options(alias="SerialNumber"))
    firmware_version: str = field(metadata=field_options(alias="Firmwareversion"))
    smart_agent_version: str = field(metadata=field_options(alias="SmartAgentversion"))
    firewall_version: str = field(metadata=field_options(alias="FirewallVersion"))
    vpn_version: str | None = field(metadata=field_options(alias="VPNVersion"))
    other_software_version: str = field(
        metadata=field_options(alias="OthersoftwareVersion")
    )
    hardware_version: str = field(metadata=field_options(alias="Hardwareversion"))
    other_hardware_version: str | None = field(
        metadata=field_options(alias="Otherhardwareversion")
    )
    device_name: str = field(metadata=field_options(alias="DeviceName"))
    device_mode: DeviceMode = field(metadata=field_options(alias="DeviceMode"))

    @classmethod
    def __pre_deserialize__(cls, d: dict[str, Any]) -> dict[str, Any]:
        """Pre deserialize hook."""
        if d["VPNVersion"] == "N/A":
            d["VPNVersion"] = None
        if d["Otherhardwareversion"] == "N/A":
            d["Otherhardwareversion"] = None
        return {**d, "DeviceMode": int(d["DeviceMode"])}


@dataclass
class TrafficMeterStatistics(DataClassDictMixin):
    """Represents the traffic meter statistics."""

    connection_time_today: int = field(
        metadata=field_options(alias="NewTodayConnectionTime")
    )
    upload_today: float = field(metadata=field_options(alias="NewTodayUpload"))
    download_today: float = field(metadata=field_options(alias="NewTodayDownload"))
    connection_time_yesterday: int = field(
        metadata=field_options(alias="NewYesterdayConnectionTime")
    )
    upload_yesterday: float = field(metadata=field_options(alias="NewYesterdayUpload"))
    download_yesterday: float = field(
        metadata=field_options(alias="NewYesterdayDownload")
    )
    connection_time_week: int = field(
        metadata=field_options(alias="NewWeekConnectionTime")
    )
    total_upload_week: float
    average_upload_week: float
    total_download_week: float
    average_download_week: float
    connection_time_month: int = field(
        metadata=field_options(alias="NewMonthConnectionTime")
    )
    total_upload_month: float
    average_upload_month: float
    total_download_month: float
    average_download_month: float
    connection_time_last_month: int = field(
        metadata=field_options(alias="NewLastMonthConnectionTime")
    )
    total_upload_last_month: float
    average_upload_last_month: float
    total_download_last_month: float
    average_download_last_month: float

    @classmethod
    def __pre_deserialize__(cls, d: dict[str, Any]) -> dict[str, Any]:
        """Pre deserialize hook."""
        total_upload_week, average_upload_week = statistics_to_data(d["NewWeekUpload"])
        total_download_week, average_download_week = statistics_to_data(
            d["NewWeekDownload"]
        )
        total_upload_month, average_upload_month = statistics_to_data(
            d["NewMonthUpload"]
        )
        total_download_month, average_download_month = statistics_to_data(
            d["NewMonthDownload"]
        )
        total_upload_last_month, average_upload_last_month = statistics_to_data(
            d["NewLastMonthUpload"]
        )
        total_download_last_month, average_download_last_month = statistics_to_data(
            d["NewLastMonthDownload"]
        )
        return {
            **d,
            "NewTodayConnectionTime": time_to_minutes(d["NewTodayConnectionTime"]),
            "NewYesterdayConnectionTime": time_to_minutes(
                d["NewYesterdayConnectionTime"]
            ),
            "NewWeekConnectionTime": time_to_minutes(d["NewWeekConnectionTime"]),
            "total_upload_week": total_upload_week,
            "average_upload_week": average_upload_week,
            "total_download_week": total_download_week,
            "average_download_week": average_download_week,
            "NewMonthConnectionTime": time_to_minutes(d["NewMonthConnectionTime"]),
            "total_upload_month": total_upload_month,
            "average_upload_month": average_upload_month,
            "total_download_month": total_download_month,
            "average_download_month": average_download_month,
            "NewLastMonthConnectionTime": time_to_minutes(
                d["NewLastMonthConnectionTime"]
            ),
            "total_upload_last_month": total_upload_last_month,
            "average_upload_last_month": average_upload_last_month,
            "total_download_last_month": total_download_last_month,
            "average_download_last_month": average_download_last_month,
        }


@dataclass
class SystemInfo(DataClassDictMixin):
    """Represents the system information."""

    cpu_usage: int = field(metadata=field_options(alias="NewCPUUtilization"))
    memory_usage: int = field(metadata=field_options(alias="NewMemoryUtilization"))
